'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.Doughnut = Doughnut;
exports.Pie = Pie;
exports.Line = Line;
exports.Bar = Bar;
exports.Radar = Radar;
exports.Polar = Polar;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _chartJs = require('chart.js');

var _chartJs2 = _interopRequireDefault(_chartJs);

var _utilsDeepEqual = require('./utils/deepEqual');

var _utilsDeepEqual2 = _interopRequireDefault(_utilsDeepEqual);

var ChartComponent = _react2['default'].createClass({

	displayName: 'ChartComponent',

	propTypes: {
		data: _react.PropTypes.object.isRequired,
		height: _react.PropTypes.number,
		legend: _react.PropTypes.object,
		options: _react.PropTypes.object,
		redraw: _react.PropTypes.bool,
		type: _react.PropTypes.oneOf(['doughnut', 'pie', 'line', 'bar', 'radar', 'polarArea']),
		width: _react.PropTypes.number
	},

	getDefaultProps: function getDefaultProps() {
		return {
			legend: {
				display: true,
				position: 'bottom'
			},
			type: 'doughnut',
			height: 200,
			width: 200,
			redraw: false
		};
	},

	componentWillMount: function componentWillMount() {
		this.chart_instance = undefined;
	},

	componentDidMount: function componentDidMount() {
		this.renderChart();
	},

	componentDidUpdate: function componentDidUpdate() {
		if (this.props.redraw) {
			this.chart_instance.destroy();
			this.renderChart();
		} else {
			this.updateChart();
		}
	},

	_objectWithoutProperties: function _objectWithoutProperties(obj, keys) {
		var target = {};
		for (var i in obj) {
			if (keys.indexOf(i) >= 0) continue;
			if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
			target[i] = obj[i];
		}
		return target;
	},

	shouldComponentUpdate: function shouldComponentUpdate(nextProps, nextState) {
		var compareNext = this._objectWithoutProperties(nextProps, ["id", "width", "height"]);
		var compareNow = this._objectWithoutProperties(this.props, ["id", "width", "height"]);
		return !(0, _utilsDeepEqual2['default'])(compareNext, compareNow, { strict: true });
	},

	componentWillUnmount: function componentWillUnmount() {
		this.chart_instance.destroy();
	},

	updateChart: function updateChart() {
		var _this = this;

		var _props = this.props;
		var data = _props.data;
		var options = _props.options;

		this.chart_instance.options.scales.xAxes[0].ticks.min = options.scales.xAxes[0].ticks.min;
		this.chart_instance.options.scales.xAxes[0].ticks.max = options.scales.xAxes[0].ticks.max;
		this.chart_instance.options.scales.xAxes[0].fixedBarWidth = options.scales.xAxes[0].fixedBarWidth;

		data.datasets.forEach(function (dataset, index) {
			_this.chart_instance.data.datasets[index].backgroundColor = dataset.backgroundColor;
		});

		this.chart_instance.update();
	},

	renderChart: function renderChart() {
		var _props2 = this.props;
		var data = _props2.data;
		var options = _props2.options;
		var legend = _props2.legend;
		var type = _props2.type;

		var node = _reactDom2['default'].findDOMNode(this);

		this.chart_instance = new _chartJs2['default'](node, {
			type: type,
			data: data,
			options: options
		});
	},

	render: function render() {
		var _props3 = this.props;
		var height = _props3.height;
		var width = _props3.width;

		return _react2['default'].createElement('canvas', {
			height: height,
			width: width
		});
	}
});

exports['default'] = ChartComponent;

function Doughnut(props) {
	return _react2['default'].createElement(ChartComponent, _extends({}, props, { type: 'doughnut' }));
}

function Pie(props) {
	return _react2['default'].createElement(ChartComponent, _extends({}, props, { type: 'pie' }));
}

function Line(props) {
	return _react2['default'].createElement(ChartComponent, _extends({}, props, { type: 'line' }));
}

function Bar(props) {
	return _react2['default'].createElement(ChartComponent, _extends({}, props, { type: 'bar' }));
}

function Radar(props) {
	return _react2['default'].createElement(ChartComponent, _extends({}, props, { type: 'radar' }));
}

function Polar(props) {
	return _react2['default'].createElement(ChartComponent, _extends({}, props, { type: 'polarArea' }));
}